/********************************** (C) COPYRIGHT *******************************
* File Name          : main.c
* Author             : WCH
* Version            : V1.0.0
* Date               : 2021/06/06
* Description        : Main program body.
*********************************************************************************
* Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
* Attention: This software (modified or not) and binary are used for 
* microcontroller manufactured by Nanjing Qinheng Microelectronics.
*******************************************************************************/

/*
 *@Note
 *Hardware NSS mode, master/slave mode, data transceiver:
 *
 *This example demonstrates that in hardware NSS mode, the Master and
 *Slave can transmit and receive in full duplex at the same time.
 *Note: The two boards download the Master and Slave programs respectively,
 *and power on at the same time.It is recommended that the NSS pin be 
 *connected to a 10 pull-up resistor.
 *
 *    Hardware connection:
 *              PB12 -- PB12
 *              PB13 -- PB13
 *              PB14 -- PB14
 *              PB15 -- PB15
 *
*/

#include "debug.h"
#include "string.h"

/* SPI Mode Definition */
#define HOST_MODE   0
#define SLAVE_MODE   1

/* SPI Communication Mode Selection */
//#define SPI_MODE   HOST_MODE
#define SPI_MODE   SLAVE_MODE

/* Global define */
#define Size 18

/* Global Variable */

u8 TxData[Size] = { 0x01, 0x02, 0x03, 0x04, 0x05, 0x06,
                    0x11, 0x12, 0x13, 0x14, 0x15, 0x16,
                    0x21, 0x22, 0x23, 0x24, 0x25, 0x26 };
u8 RxData[Size];

/*********************************************************************
 * @fn      SPI_FullDuplex_Init
 *
 * @brief   Configuring the SPI for full-duplex communication.
 *
 * @return  none
 */
void SPI_FullDuplex_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure={0};
	SPI_InitTypeDef  SPI_InitStructure={0};

	RCC_APB2PeriphClockCmd( RCC_APB2Periph_GPIOB , ENABLE );
	RCC_APB1PeriphClockCmd( RCC_APB1Periph_SPI2, ENABLE );

#if (SPI_MODE == HOST_MODE)
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init( GPIOB, &GPIO_InitStructure );

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init( GPIOB, &GPIO_InitStructure );

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_14;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init( GPIOB, &GPIO_InitStructure );

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init( GPIOB, &GPIO_InitStructure );

#elif (SPI_MODE == SLAVE_MODE)
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;
	GPIO_Init( GPIOB, &GPIO_InitStructure );

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init( GPIOB, &GPIO_InitStructure );

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_14;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init( GPIOB, &GPIO_InitStructure );

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init( GPIOB, &GPIO_InitStructure );

#endif

#if (SPI_MODE == HOST_MODE)
	SPI_SSOutputCmd( SPI2, ENABLE );
#endif

	SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;

#if (SPI_MODE == HOST_MODE)
	SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
#elif (SPI_MODE == SLAVE_MODE)
	SPI_InitStructure.SPI_Mode = SPI_Mode_Slave;
#endif

	SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
	SPI_InitStructure.SPI_CPOL = SPI_CPOL_High;
	SPI_InitStructure.SPI_CPHA = SPI_CPHA_2Edge;
	SPI_InitStructure.SPI_NSS = SPI_NSS_Hard;
	SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_32;
	SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
	SPI_InitStructure.SPI_CRCPolynomial = 7;
	SPI_Init( SPI2, &SPI_InitStructure );

	SPI_Cmd( SPI2, ENABLE );
}

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
	u8 i=0;
	u8 j=0;
	u8 value;

	SystemCoreClockUpdate();
	Delay_Init();
	USART_Printf_Init(115200);
	printf("SystemClk:%d\r\n",SystemCoreClock);
	printf( "ChipID:%08x\r\n", DBGMCU_GetCHIPID() );

#if (SPI_MODE == SLAVE_MODE)
  printf("Slave Mode\r\n");
	Delay_Ms(1000);

#endif

	SPI_FullDuplex_Init();

#if (SPI_MODE == HOST_MODE)
    printf("Host Mode\r\n");
	Delay_Ms(2000);

#endif

	while(1)
	{
          if( SPI_I2S_GetFlagStatus( SPI2, SPI_I2S_FLAG_TXE ) != RESET )
          {
              SPI_I2S_SendData( SPI2, 0xA5 );
          }

          if( SPI_I2S_GetFlagStatus( SPI2, SPI_I2S_FLAG_RXNE ) != RESET )
          {
              value = SPI_I2S_ReceiveData( SPI2 );
          }



//		while( ( i<18 ) || ( j<18 ))
//		{
//			if( i<18 )
//			{
//				if( SPI_I2S_GetFlagStatus( SPI2, SPI_I2S_FLAG_TXE ) != RESET )
//				{
//					SPI_I2S_SendData( SPI2, TxData[i] );
//					i++;
//				}
//			}
//
//			if( j<18 )
//			{
//				if( SPI_I2S_GetFlagStatus( SPI2, SPI_I2S_FLAG_RXNE ) != RESET )
//				{
//					RxData[j] = SPI_I2S_ReceiveData( SPI2 );
//					j++;
//				}
//			}
//		}
//
//		SPI_Cmd( SPI2, DISABLE );
//
//		value = memcmp( TxData, RxData, Size );
//
//		if( value == 0 )
//		{
//			printf( "Same\r\n" );
//		}
//		else
//		{
//			printf( "Different\r\n" );
//		}
//
//		while(1);
	}
}
